//---------------------------------------------------------------------------

#pragma hdrstop

#include "Animal.h"

int Animal::AnimalCounter = 0;

#if SEED
int32_t Aseed = (int32_t) SEED;
#else
int32_t Aseed = (int32_t)time(0);
#endif
StochasticLib1 sto1animal(Aseed);

//---------------------------------------------------------------------------
#pragma package(smart_init)

//------------------------------------------------------//Constructor definition
Animal::Animal()
{
id = AnimalCounter; AnimalCounter++; // unique identifier for each animal
// start at random location
loc.x = sto1animal.IRandom(0,parameters->getXmax()-1);
loc.y = sto1animal.IRandom(0,parameters->getYmax()-1);
Pdisperse = sto1animal.Bernoulli(0.5); // equal prob of each allele
Nsteps = 0;
}
//------------------------------------------------------------------------------
Animal::Animal(const Animal* parent, double Pmut)
{
id = AnimalCounter; AnimalCounter++; // unique identifier for each animal
loc = parent->loc;
Nsteps = 0;

// inherit dispersal probability
Pdisperse = parent->Pdisperse;
if (sto1animal.Random() < Pmut) {
  if (Pdisperse == 0) Pdisperse = 1; else Pdisperse = 0;
}

}
//------------------------------------------------------------------------------

int Animal::get_id() { return id; }
locn Animal::get_locn() { return loc; }
double Animal::get_Pdisperse() { return Pdisperse; }
int Animal::get_Nsteps() { return Nsteps; }

genome Animal::get_genome() {
genome g;
g.gene0 = Pdisperse; g.gene1 = PstopInt; g.gene2 = PstopSlope;
return g;
}

void Animal::ResetGenome(genome g) {
// Resets the values of the animal's genes
Pdisperse = g.gene0;  PstopInt = g.gene1;  PstopSlope = g.gene2;
}

locn Animal::Move(Landscape* pLand, double dispMort)
{
locn dest,bnds;
double prey,logPstop,Pstop,rr;
bool died = 0;
Square* sq;
int i,j,K;

dest = loc;

if (sto1animal.Random() < dispMort) died = 1;
else {
  // generate a random number for the animal in range 0 to 1
  // landscape determines stopping square on basis of random number for several options
  rr = sto1animal.Random();
  switch ( parameters->getDispMethod()) {
    case 0: // global random
	    dest.x = sto1animal.IRandom(0,parameters->getXmax()-1);
	    dest.y = sto1animal.IRandom(0,parameters->getYmax()-1);
      Nsteps++;
      break;
    case 1: // global in proportion to Kmean
      dest = pLand->randomSquare(rr);
      break;
    case 2: // global in proportion to 1 - N/K
      dest = pLand->attractiveSquare(rr,0);
      break;
    case 3: // global in proportion to 1/exp(N/K)
      dest = pLand->attractiveSquare(rr,1);
      break;
    case 4: // global in proportion to exp((K-N)/K)/(1+exp((K-N)/K))
      dest = pLand->attractiveSquare(rr,2);
      break;
    case 5: // nearest neighbour (treated as a torus)
      while (dest.x == loc.x && dest.y == loc.y) {
        dest.x = loc.x + sto1animal.IRandom(0,2) - 1;
        dest.y = loc.y + sto1animal.IRandom(0,2) - 1;
      }
	    if (dest.x < 0) dest.x = parameters->getXmax()-1; if (dest.x >= parameters->getXmax()) dest.x = 0;
	    if (dest.y < 0) dest.y = parameters->getYmax()-1; if (dest.y >= parameters->getYmax()) dest.y = 0;
      break;
    default: // go to (0,0) which should be obvious something is wrong!
      dest.x = 0; dest.y = 0;
    ;
  }
}

if (died) {
  dest.x = dest.y = -999;
}
else {
  if (dest.x < 0) dest.x = 0;
  if (dest.x >= parameters->getXmax()) dest.x = parameters->getXmax()-1;
  if (dest.y < 0) dest.y = 0;
  if (dest.y >= parameters->getYmax()) dest.y = parameters->getYmax()-1;
  loc = dest;
}

return dest;
}
//------------------------------------------------------------------------------
