/*------------------------------------------------------------------------------

Beetle01 class Animal

The haploid Animal class has a genome of three genes, but only the first (integer)
gene is used in this application, to determine whether an individual is the resident
or dispersing morph.
Other than at model initialisation, an animal inherits its genome from its single
parent with a small probability of mutation to the other allele.

Dispersing morphs exhibit the behaviour to make a single-step move to a new square
within the landscape selected on the basis of one of a number of optional dispersal
methods as documented within the Move() function.

Author: Steve Palmer, University of Aberdeen

Last updated: 8 December 2011

------------------------------------------------------------------------------*/

//---------------------------------------------------------------------------

#ifndef AnimalH
#define AnimalH
//---------------------------------------------------------------------------

#include <vector>
//using namespace std;

#include "stocc.h"
#include "Parameters.h"
#include "Landscape.h"
extern Parameters* parameters;
struct genome { int gene0; double gene1; double gene2; };

//==============================================================================
class Animal
{
public:
  static int AnimalCounter; // Used to create ID, held by class not members of class
  Animal(); // Animal constructor
  Animal(const Animal*,double); // Animal copy constructor.
  locn Move(Landscape*,double);
  int get_id();
  locn get_locn();
  double get_Pdisperse();
  genome get_genome();
  int get_Nsteps();
  void ResetGenome(genome);

private:
	//input parameters
	Parameters* inputParam;
  int id;
  locn loc;
  int Pdisperse;   // 'gene' controlling probability of dispersal:
  // 0 = small-winged (resident) 1 = large-winged (disperser)
  double PstopInt;    // ) 'genes' controlling probability of stopping as fn of
  double PstopSlope;  // ) prey density - intercept and slope in logit space
  int Nsteps;         // no. of steps taken during dispersal
 };

//==============================================================================


#endif
