/*------------------------------------------------------------------------------

Beetle01 model, main() function

Sets up the landscape and beetle population (for historical reasons regarded as
'predators').
Runs the model for the specified number of generations.
Produces output files of parameters applied and summary population data.

Author: Steve Palmer, University of Aberdeen

Last updated: 7 December 2011

------------------------------------------------------------------------------*/

//---------------------------------------------------------------------------

#include <iostream.h>
#include <stdlib.h>

#pragma hdrstop

#include "Parameters.h"
#include "Landscape.h"
#include "Predators.h"
//#include "Samples.h"

// global pointers to landscape (prey), predator population and parameters
Landscape* pLandscape;
PredPop* pPredators;
Parameters* parameters;
//SampleSet* pSamples;

int predgen = 0; // predator generation number

double kmean;
double ksd;
int kmin;

ofstream outparam("Params.csv"); // parameter summary data
ofstream outsumm("Summary.csv"); // summary data file

//---------------------------------------------------------------------------

#pragma argsused
int main(int argc, char* argv[])
{
int i,t0,t1,Nruns,juvs;
int repl = 0;
PredStats p;

t0 = time(0);

// write parameter file headers
outparam<<"Repl,Xmax,Ymax,Kmean,Ksd,Kmin,TempMthd,TempSD,DispMthd,lambda,b,dispMort"<<endl;
// write summary data file headers
outsumm<<"Repl,Gen,Njuvs,Dispersers,Residents,PredPDispMean,PredPDispSD"
  <<endl;

parameters = new Parameters();
kmean = parameters->getMU();
ksd = parameters->getSdSpatial();
kmin = parameters->getKmin();

// parameter control loops

pLandscape = new Landscape();

//pPredators = new PredPop(PRED_START,PRED_LAMBDA,PRED_B,PRED_MORT);
pPredators = new PredPop(parameters->getPredstart(),parameters->getPredLambda(),
  parameters->getPredB(),parameters->getPredMort());
pPredators->InitialisePreds(pLandscape);
p = pPredators->GetStats(pLandscape);

  // write parameter values to file
  outparam<<repl<<","<<parameters->getXmax()<<","<<parameters->getYmax()
    <<","<<parameters->getMU()<<","<<parameters->getSdSpatial()<<","<<parameters->getKmin()
    <<","<<parameters->getTempMethod()<<","<<parameters->getSdTemp()
    <<","<<parameters->getDispMethod()<<","<<pPredators->get_lambda()
    <<","<<pPredators->get_b()<<","<<pPredators->get_dispMort()<<endl;
  // write initial summary data to file
  outsumm<<repl<<",0,0,"<<p.dispersers<<","<<p.residents<<","
    <<p.meanDisp<<","<<p.sdDisp<<","
    <<endl;

  for (int i = 0; i < parameters->getPredGen(); i++) {
    predgen++;
#if !DEBUG
    if (predgen%10 == 0) cout<<" "<<predgen;
#endif
    juvs = pPredators->Breed(pLandscape,parameters->getPredPmut(),predgen);
    if (juvs < 1) { // predator population has become extinct
      p.n = 0; p.meanDisp = 0.0; p.sdDisp = 0.0; p.dispersers = 0; p.meanNsteps = 0;
      i = 99999999; // to exit for loop
    }
    else {
	    pLandscape->update_K(parameters->getTempMethod(),parameters->getSdTemp());
      pLandscape->resetPredStats();
      pPredators->Disperse(pLandscape);
      p = pPredators->GetStats(pLandscape);
    }
    if (predgen >= parameters->getSpatialSummStart()) {
      pLandscape->incrTotals();
    }
    // write summary data to file
    if (predgen%parameters->getSummaryFreq() == 0) {
      outsumm<<repl<<","<<predgen<<","<<juvs<<","<<p.dispersers<<","<<p.residents<<","
        <<p.meanDisp<<","<<p.sdDisp<<","
        <<endl;
    }
  }

if (pLandscape != 0) {
  pLandscape->spatialSummary(repl);
  delete pLandscape; pLandscape = 0;
}
if (pPredators != 0) {
  delete pPredators; pPredators = 0;
}


outparam.close();
outsumm.close();
//pSamples->CloseFiles();

t1 = time(0);
cout<<endl<<endl<<"***** Elapsed time "<<t1-t0<<" seconds"<<endl<<endl;

cout<<"*****"<<endl;
cout<<"*****Simulation completed - enter any number to terminate program"<<endl;
cout<<"*****"<<endl;
cin>>i;

return 0;
}


//---------------------------------------------------------------------------
