//---------------------------------------------------------------------------

#pragma hdrstop

#include "Parameters.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)

Parameters::Parameters(void)
{
	//default values for the parameters taken from the original constants

  // full run
	XMAX = 20; // max. X value (max 400)
	YMAX = 20; // max. Y value (max 100)
	PREDGENS = 500;   // no. of predator generations per run
	MU = 50;  // mean carrying capacity
	SDSPATIAL = 10;  // spatial s.d. in carrying capacity
	KMIN = 10;  // minimum permitted carrying capacity
	SDTEMP = 5.0;  // temporal s.d. in carrying capacity

  /*
  // debug run
	XMAX = 3; // max. X value (max 400)
	YMAX = 2; // max. Y value (max 100)
	PREDGENS = 1;   // no. of predator generations per run
	MU = 4;  // mean carrying capacity
	SDSPATIAL = 2;  // spatial s.d. in carrying capacity
	KMIN = 3;  // minimum permitted carrying capacity
	SDTEMP = 5.0;  // temporal s.d. in carrying capacity
  */

//	XMAX = 20; // max. X value (max 400)
//	YMAX = 20; // max. Y value (max 100)
	PIX = 10; // not used by batch version
	// Control parameters
	TEMP_METHOD = 0;  // Method for temporal vairation in carrying capacity:
// 0 = Poisson, 1 = normal
	DISP_METHOD = 0;  // Dispersal method:
// 0 = global random
// 1 = global in proportion to Kmean
// 2 = global in proportion to 1 - N/K
// 3 = global in proportion to 1/exp(N/K)
// 4 = global in proportion to exp((K-N)/K)/(1+exp((K-N)/K))
// 5 = nearest neighbour random (torus)
	DISPLAYFREQ = 20;    // display map refresh frequency - not used by batch version
	SUMMARYFREQ = 1;    // generation summary file output frequency
	SPATIALSUMMSTART = 400;  // generation to start spatial summary data

  // Predator parameters
	PRED_START  = XMAX * YMAX * MU;   // predator initial population
// set to approx. XMAX * YMAX * MU
	PRED_LAMBDA  = 3.0; // predator intrinsic growth rate
	PRED_B  = 1.0;      // predator competition coefficient
	PRED_MORT  = 0.00;  // predator dispersal mortality rate
	PRED_PMUT  = 0.001;  // predator mutation probability (z)
}

Parameters::Parameters(Parameters* p)
{
	XMAX = p->getXmax(); // max. X value (max 400)
	YMAX = p->getYmax(); // max. Y value (max 100)
	PIX = p->getPix();
	// Control parameters
	PREDGENS = p->getPredGen();   // no. of predator generations per run
	TEMP_METHOD = p->getTempMethod();  // Method for temporal vairation in carrying capacity:
// 0 = Poisson, 1 = normal
	DISP_METHOD = p->getDispMethod();  // Dispersal method:
// 0 = global random, 1 = global in proportion to K, 2 = nearest neighbour (torus)
	DISPLAYFREQ = p->getDispFreq();    // display map refresh frequency
	SUMMARYFREQ = p->getSummaryFreq();    // generation summary file output frequency
	SPATIALSUMMSTART = p->getSpatialSummStart();  // generation to start spatial summary data

// Landscape parameters
	MU = p->getMU();  // mean carrying capacity
	SDSPATIAL = p->getSdSpatial();  // spatial s.d. in carrying capacity
	KMIN = p->getKmin();  // minimum permitted carrying capacity
	SDTEMP = p->getSdTemp();  // temporal s.d. in carrying capacity

// Predator parameters
	PRED_START  = XMAX * YMAX * MU;   // predator initial population
// set to approx. XMAX * YMAX * MU
	PRED_LAMBDA  = p->getPredLambda(); // predator intrinsic growth rate
	PRED_B  = p->getPredB();      // predator competition coefficient
	PRED_MORT  = p->getPredMort();  // predator dispersal mortality rate
	PRED_PMUT  = p->getPredPmut();  // predator mutation probability (z)
}

int Parameters::getXmax() { return this->XMAX; }
int Parameters::getYmax() {	return this->YMAX; }
int Parameters::getPix() { return this->PIX; }
int Parameters::getPredGen() { return this->PREDGENS; }
int Parameters::getTempMethod() { return this->TEMP_METHOD; }
int Parameters::getDispMethod() { return this->DISP_METHOD; }
int Parameters::getDispFreq() { return this->DISPLAYFREQ; }
int Parameters::getSummaryFreq() { return this->SUMMARYFREQ; }
int Parameters::getSpatialSummStart() {	return this->SPATIALSUMMSTART; }
double Parameters::getMU() { return this->MU; }
double Parameters::getSdSpatial() {	return this->SDSPATIAL; }
int Parameters::getKmin() { return this->KMIN; }
double Parameters::getSdTemp() { return this->SDTEMP; }
int Parameters::getPredstart() { return this->PRED_START; }
double Parameters::getPredLambda() { return this->PRED_LAMBDA; }
double Parameters::getPredB() { return this->PRED_B; }
double Parameters::getPredMort() { return this->PRED_MORT; }
double Parameters::getPredPmut() { return this->PRED_PMUT; }

void Parameters::setXmax(int maxX) { this->XMAX = maxX; }
void Parameters::setYmax(int maxY) { this->YMAX = maxY; }
void Parameters::setPix(int pixelSize) { this->PIX = pixelSize; }
void Parameters::setPredGen(int generations) { this->PREDGENS = generations; }
void Parameters::setTempMethod(int tempMeth) { this->TEMP_METHOD = tempMeth; }
void Parameters::setDispMethod(int dispersalmethod)
{ this->DISP_METHOD = dispersalmethod; }
void Parameters::setDispFreq(int refreshFreq) { this->DISPLAYFREQ = refreshFreq; }
void Parameters::setSummaryFreq(int outputFrq) { this->SUMMARYFREQ = outputFrq; }
void Parameters::setSpatialSummStart(int gen) {	this->SPATIALSUMMSTART = gen; }
void Parameters::setMU(double mean) {	this->MU = mean; }
void Parameters::setSdSpatial(double spatialSD) {	this->SDSPATIAL = spatialSD; }
void Parameters::setKmin(int minSD) {	this->KMIN = minSD; }
void Parameters::setSdTemp(double temporalSD) {	this->SDTEMP = temporalSD; }
void Parameters::setPredstart(int initialPop)
{
//	this->PRED_START = initialPop;
	this->PRED_START = this->XMAX * this->YMAX * this->MU;
}
void Parameters::setPredLambda(double growthRate) {	this->PRED_LAMBDA = growthRate; }
void Parameters::setPredB(double competitionCoeff) { this->PRED_B = competitionCoeff; }
void Parameters::setPredMort(double mortalityRate) { this->PRED_MORT = mortalityRate; }
void Parameters::setPredPmut(double mutationProb) {	this->PRED_PMUT = mutationProb; }
